@@ -25,7 +25,7 @@ geom_ignore <- function(...) {
 #'
 #' The "area ridges" are for use in `mcmc_areas()`. The scale of 1 and the
 #' identity statistic prevent the ridges from overlapping.
-#' `geom_density_ridges2` draws closed polygons.
+#' `geom_density_ridges2()` draws closed polygons.
 #'
 #' @importFrom ggridges geom_density_ridges geom_density_ridges2
 #' @noRd
@@ -64,12 +64,26 @@ dont_expand_axes <- function() {
 }
 force_axes_in_facets <- function() {
   thm <- bayesplot_theme_get()
-  annotate("segment",
-           x = c(-Inf, -Inf), xend = c(Inf,-Inf),
-           y = c(-Inf,-Inf), yend = c(-Inf, Inf),
-           color = thm$axis.line$colour %||% "black",
-           size = thm$axis.line$size %||% 0.5)
+  annotate(
+    "segment",
+    x = c(-Inf, -Inf), xend = c(Inf,-Inf),
+    y = c(-Inf,-Inf), yend = c(-Inf, Inf),
+    color = thm$axis.line$colour %||% thm$line$colour %||% "black",
+    size = thm$axis.line$size %||% thm$line$size %||% 0.5
+  )
 }
+
+force_x_axis_in_facets <- function() {
+  thm <- bayesplot_theme_get()
+  annotate(
+    "segment",
+    x = -Inf, xend = Inf,
+    y = -Inf, yend = -Inf,
+    color = thm$axis.line$colour %||% thm$line$colour %||% "black",
+    size = thm$axis.line$size %||% thm$line$size %||% 0.5
+  )
+}
+
 no_legend_spacing <- function() {
   theme(legend.spacing.y = unit(0, "cm"))
 }
@@ -83,8 +97,57 @@ space_legend_keys <- function(relative_size = 2, color = "white") {
 
 # set aesthetic mapping for histograms depending on freq argument
 set_hist_aes <- function(freq = TRUE, ...) {
-  if (freq)
+  if (freq) {
     aes_(x = ~ value, ...)
-  else
-    aes_(x = ~ value, y = ~ ..density.., ...)
+  } else {
+    aes_(x = ~ value, y = ~ stat(density), ...)
+  }
 }
+
+scale_color_ppc <-
+  function(name = NULL,
+           values = NULL,
+           labels = NULL,
+           ...) {
+    scale_color_manual(
+      name = name %||% "",
+      values = values %||% get_color(c("dh", "lh")),
+      labels = labels %||% c(y_label(), yrep_label()),
+      ...
+    )
+  }
+
+scale_fill_ppc <-
+  function(name = NULL,
+           values = NULL,
+           labels = NULL,
+           ...) {
+    scale_fill_manual(
+      name = name %||% "",
+      values = values %||% get_color(c("d", "l")),
+      labels = labels %||% c(y_label(), yrep_label()),
+      ...
+    )
+  }
+
+scale_color_ppd <-
+  function(name = NULL,
+           values = get_color("mh"),
+           labels = ypred_label(),
+           ...) {
+    scale_color_ppc(name = name,
+                    values = values,
+                    labels = labels,
+                    ...)
+  }
+
+scale_fill_ppd <-
+  function(name = NULL,
+           values = get_color("m"),
+           labels = ypred_label(),
+           ...) {
+    scale_fill_ppc(name = name,
+                   values = values,
+                   labels = labels,
+                   ...)
+  }
