@@ -25,6 +25,9 @@
 #'   `"equal height"` are scaled using `height*sqrt(height)`
 #' @param point_est The point estimate to show. Either `"median"` (the
 #'   default), `"mean"`, or `"none"`.
+#' @param inner_size,outer_size For `mcmc_intervals()`, the size of
+#'   the inner and interval segments, respectively.
+#' @param point_size For `mcmc_intervals()`, the size of point estimate.
 #' @param rhat An optional numeric vector of R-hat estimates, with one element
 #'   per parameter included in `x`. If `rhat` is provided, the intervals/areas
 #'   and point estimates in the resulting plot are colored based on R-hat value.
@@ -52,6 +55,10 @@
 #'
 #' @examples
 #' set.seed(9262017)
+#'
+#' # load ggplot2 to use its functions to modify our plots
+#' library(ggplot2)
+#'
 #' # some parameter draws to use for demonstration
 #' x <- example_mcmc_draws(params = 6)
 #' dim(x)
@@ -61,18 +68,44 @@
 #' mcmc_intervals(x)
 #' mcmc_intervals(x, pars = c("beta[1]", "beta[2]"))
 #' mcmc_areas(x, regex_pars = "beta\\[[1-3]\\]",  prob = 0.8) +
-#'  ggplot2::labs(
+#'  labs(
 #'    title = "Posterior distributions",
 #'    subtitle = "with medians and 80% intervals"
 #'  )
 #'
 #' color_scheme_set("red")
-#' mcmc_areas(
+#' p <- mcmc_areas(
 #'    x,
 #'    pars = c("alpha", "beta[4]"),
 #'    prob = 2/3,
 #'    prob_outer = 0.9,
-#'    point_est = "mean"
+#'    point_est = "mean",
+#'    border_size = 1.5 # make the ridgelines fatter
+#' )
+#' plot(p)
+#'
+#' # control spacing at top and bottom of plot
+#' # see ?ggplot2::expansion
+#' p + scale_y_discrete(
+#'   limits = c("beta[4]", "alpha"),
+#'   expand = expansion(add = c(1, 2))
+#' )
+#' p + scale_y_discrete(
+#'   limits = c("beta[4]", "alpha"),
+#'   expand = expansion(add = c(.1, .3))
+#' )
+#'
+#' # relabel parameters
+#' p + scale_y_discrete(
+#'   labels = c("alpha" = "param label 1",
+#'              "beta[4]" = "param label 2")
+#')
+#'
+#' # relabel parameters and define the order
+#' p + scale_y_discrete(
+#'   labels = c("alpha" = "param label 1",
+#'              "beta[4]" = "param label 2"),
+#'   limits = c("beta[4]", "alpha")
 #' )
 #'
 #' # color by rhat value
@@ -94,22 +127,22 @@
 #' b3 <- c("beta[1]", "beta[2]", "beta[3]")
 #'
 #' mcmc_areas(x, pars = b3, area_method = "equal area") +
-#'   ggplot2::labs(
+#'   labs(
 #'     title = "Curves have same area",
-#'     subtitle =
-#'       "A wide, uncertain interval is spread thin when areas are equal")
+#'     subtitle = "A wide, uncertain interval is spread thin when areas are equal"
+#'    )
 #'
 #' mcmc_areas(x, pars = b3, area_method = "equal height") +
-#'   ggplot2::labs(
+#'   labs(
 #'     title = "Curves have same maximum height",
-#'     subtitle =
-#'       "Local curvature is clearer but more uncertain curves use more area")
+#'     subtitle = "Local curvature is clearer but more uncertain curves use more area"
+#'   )
 #'
 #' mcmc_areas(x, pars = b3, area_method = "scaled height") +
-#'   ggplot2::labs(
+#'   labs(
 #'     title = "Same maximum heights but heights scaled by square-root",
-#'     subtitle =
-#'       "Compromise: Local curvature is accentuated and less area is used")
+#'     subtitle = "Compromise: Local curvature is accentuated and less area is used"
+#'    )
 #'
 #' \donttest{
 #' # apply transformations
@@ -144,8 +177,8 @@
 #' # Example of hierarchically related parameters
 #' # plotted with ridgelines
 #' m <- shinystan::eight_schools@posterior_sample
-#' mcmc_areas_ridges(m, pars = "mu", regex_pars = "theta") +
-#'  ggplot2::ggtitle("Treatment effect on eight schools (Rubin, 1981)")
+#' mcmc_areas_ridges(m, pars = "mu", regex_pars = "theta", border_size = 0.75) +
+#'   ggtitle("Treatment effect on eight schools (Rubin, 1981)")
 #' }
 #'
 NULL
@@ -160,6 +193,9 @@ mcmc_intervals <- function(x,
                            prob = 0.5,
                            prob_outer = 0.9,
                            point_est = c("median", "mean", "none"),
+                           outer_size = 0.5,
+                           inner_size = 2,
+                           point_size = 4,
                            rhat = numeric()) {
   check_ignored_arguments(...)
 
@@ -184,17 +220,18 @@ mcmc_intervals <- function(x,
 
   args_outer <- list(
     mapping = aes_(x = ~ ll, xend = ~ hh, y = ~ parameter, yend = ~ parameter),
-    color = get_color("mid")
+    color = get_color("mid"),
+    size = outer_size
   )
   args_inner <- list(
     mapping = aes_(x = ~ l, xend = ~ h, y = ~ parameter, yend = ~ parameter),
-    size = 2,
+    size = inner_size,
     show.legend = FALSE
   )
   args_point <- list(
     mapping = aes_(x = ~ m, y = ~ parameter),
     data = data,
-    size = 4,
+    size = point_size,
     shape = 21
   )
 
@@ -245,6 +282,8 @@ mcmc_intervals <- function(x,
 
 #' @rdname MCMC-intervals
 #' @export
+#' @param border_size For `mcmc_areas()` and `mcmc_areas_ridges()`, the size of
+#'   the ridgelines.
 mcmc_areas <- function(x,
                        pars = character(),
                        regex_pars = character(),
@@ -255,6 +294,7 @@ mcmc_areas <- function(x,
                        prob_outer = 1,
                        point_est = c("median", "mean", "none"),
                        rhat = numeric(),
+                       border_size = NULL,
                        bw = NULL,
                        adjust = NULL,
                        kernel = NULL,
@@ -266,7 +306,8 @@ mcmc_areas <- function(x,
     x, pars, regex_pars, transformations,
     prob = prob, prob_outer = prob_outer,
     point_est = point_est, rhat = rhat,
-    bw = bw, adjust = adjust, kernel = kernel, n_dens = n_dens)
+    bw = bw, adjust = adjust, kernel = kernel, n_dens = n_dens
+  )
   datas <- split(data, data$interval)
 
   # Use a dummy empty dataframe if no point estimate
@@ -309,7 +350,11 @@ mcmc_areas <- function(x,
 
   datas$bottom <- datas$outer %>%
     group_by(!!! groups) %>%
-    summarise(ll = min(.data$x), hh = max(.data$x)) %>%
+    summarise(
+      ll = min(.data$x),
+      hh = max(.data$x),
+      .groups = "drop_last"
+    ) %>%
     ungroup()
 
   args_bottom <- list(
@@ -330,6 +375,12 @@ mcmc_areas <- function(x,
     fill = NA
   )
 
+  if (!is.null(border_size)) {
+    args_bottom$size <- border_size
+    args_outer$size <- border_size
+    args_inner$size <- border_size
+  }
+
   if (color_by_rhat) {
     args_bottom$mapping <- args_bottom$mapping %>%
       modify_aes_(color = ~ rhat_rating)
@@ -351,9 +402,16 @@ mcmc_areas <- function(x,
     args_outer$color <- get_color("dark")
   }
 
+  # An invisible layer that is 2.5% taller than the plotted one
+  args_outer2 <- args_outer
+  args_outer2$mapping <- args_outer2$mapping %>%
+    modify_aes_(scale = .925)
+  args_outer2$color <- NA
+
   layer_bottom <- do.call(geom_segment, args_bottom)
   layer_inner <- do.call(ggridges::geom_ridgeline, args_inner)
   layer_outer <- do.call(ggridges::geom_ridgeline, args_outer)
+  layer_outer2 <- do.call(ggridges::geom_ridgeline, args_outer2)
 
   point_geom <- if (no_point_est) {
     geom_ignore
@@ -377,12 +435,17 @@ mcmc_areas <- function(x,
     layer_inner +
     layer_point +
     layer_outer +
+    layer_outer2 +
     layer_bottom +
     scale_color +
     scale_fill +
     scale_y_discrete(
       limits = unique(rev(data$parameter)),
-      expand = expansion(add = c(0, .1), mult = c(.1, .3))) +
+      expand = expansion(
+        add = c(0, 0.5 + 1/(2 * nlevels(data$parameter))),
+        mult = c(0.05, 1/(2 * nlevels(data$parameter)))
+      )
+    ) +
     xlim(x_lim) +
     bayesplot_theme_get() +
     legend_move(ifelse(color_by_rhat, "top", "none")) +
@@ -401,6 +464,7 @@ mcmc_areas_ridges <- function(x,
                              ...,
                              prob_outer = 1,
                              prob = 1,
+                             border_size = NULL,
                              bw = NULL, adjust = NULL, kernel = NULL,
                              n_dens = NULL) {
   check_ignored_arguments(...)
@@ -431,6 +495,9 @@ mcmc_areas_ridges <- function(x,
     fill = NA,
     stat = "identity"
   )
+  if (!is.null(border_size)) {
+    args_outer$size <- border_size
+  }
 
   layer_outer <- do.call(ggridges::geom_density_ridges, args_outer)
 
@@ -469,6 +536,10 @@ mcmc_areas_ridges <- function(x,
         scale = scale,
         stat = "identity")
 
+    if (!is.null(border_size)) {
+      args_inner$size <- border_size
+    }
+
     layer_list_inner[[par_num]] <- do.call(ggridges::geom_ridgeline, args_inner)
   }
 
@@ -476,7 +547,10 @@ mcmc_areas_ridges <- function(x,
     aes_(x = ~ x, y = ~ parameter) +
     layer_outer +
     scale_y_discrete(limits = unique(rev(data$parameter)),
-                     expand = c(0.05, .6)) +
+                     expand = expansion(
+                       add = c(0, 1.4 + 1/(2 * nlevels(data$parameter))),
+                       mult = c(0.05, 1/(2 * nlevels(data$parameter)))
+                     )) +
     layer_list_inner +
     layer_vertical_line +
     scale_fill_identity() +
