@@ -7,6 +7,8 @@
 #' @family PPCs
 #'
 #' @template args-y-yrep
+#' @template args-group
+#' @template args-facet_args
 #' @param ... Currently unused.
 #' @param size,alpha For scatterplots, arguments passed to
 #'   [ggplot2::geom_point()] to control the appearance of the points. For the
@@ -28,8 +30,8 @@
 #' \describe{
 #'   \item{`ppc_error_hist()`}{
 #'    A separate histogram is plotted for the predictive errors computed from
-#'    `y` and each dataset (row) in `yrep`. For this plot `yrep`
-#'    should have only a small number of rows.
+#'    `y` and each dataset (row) in `yrep`. For this plot `yrep` should have
+#'    only a small number of rows.
 #'   }
 #'   \item{`ppc_error_hist_grouped()`}{
 #'    Like `ppc_error_hist()`, except errors are computed within levels of a
@@ -39,20 +41,18 @@
 #'   }
 #'   \item{`ppc_error_scatter()`}{
 #'    A separate scatterplot is displayed for `y` vs. the predictive errors
-#'    computed from `y` and each dataset (row) in `yrep`. For this
-#'    plot `yrep` should have only a small number of rows.
+#'    computed from `y` and each dataset (row) in `yrep`. For this plot `yrep`
+#'    should have only a small number of rows.
 #'   }
 #'   \item{`ppc_error_scatter_avg()`}{
-#'    A single scatterplot of `y` vs. the average of the errors computed
-#'    from `y` and each dataset (row) in `yrep`. For each individual
-#'    data point `y[n]` the average error is the average of the
-#'    errors for `y[n]` computed over the the draws from the posterior
-#'    predictive distribution.
+#'    A single scatterplot of `y` vs. the average of the errors computed from
+#'    `y` and each dataset (row) in `yrep`. For each individual data point
+#'    `y[n]` the average error is the average of the errors for `y[n]` computed
+#'    over the the draws from the posterior predictive distribution.
 #'   }
 #'   \item{`ppc_error_scatter_avg_vs_x()`}{
 #'    Same as `ppc_error_scatter_avg()`, except the average is plotted on the
-#'    \eqn{y}-axis and a a predictor variable `x` is plotted on the
-#'    \eqn{x}-axis.
+#'    y-axis and a predictor variable `x` is plotted on the x-axis.
 #'   }
 #'   \item{`ppc_error_binned()`}{
 #'    Intended for use with binomial data. A separate binned error plot (similar
@@ -89,8 +89,8 @@
 #' x <- example_x_data()
 #' ppc_error_scatter_avg_vs_x(y, yrep, x)
 #'
-#' # ppc_error_binned with binomial model from rstanarm
 #' \dontrun{
+#' # binned error plot with binomial model from rstanarm
 #' library(rstanarm)
 #' example("example_model", package = "rstanarm")
 #' formula(example_model)
@@ -113,31 +113,23 @@ NULL
 #' @export
 #' @template args-hist
 #' @template args-hist-freq
-#'
 ppc_error_hist <-
   function(y,
            yrep,
            ...,
+           facet_args = list(),
            binwidth = NULL,
            breaks = NULL,
            freq = TRUE) {
-    check_ignored_arguments(...)
 
-    y <- validate_y(y)
-    yrep <- validate_yrep(yrep, y)
-
-    if (nrow(yrep) == 1) {
-      errors <- data.frame(value = y - as.vector(yrep))
-      graph <- ggplot(errors, set_hist_aes(freq))
-    } else {
-      errors <- compute_errors(y, yrep)
-      graph <-
-        ggplot(melt_yrep(errors), set_hist_aes(freq)) +
-        labs(y = NULL, x = expression(italic(y) - italic(y)[rep])) +
-        facet_wrap(facets = ~ rep_id)
+    dots <- list(...)
+    if (!from_grouped(dots)) {
+      check_ignored_arguments(...)
+      dots$group <- NULL
     }
 
-    graph +
+    data <- ppc_error_data(y, yrep, group = dots$group)
+    ggplot(data, set_hist_aes(freq)) +
       geom_histogram(
         fill = get_color("l"),
         color = get_color("lh"),
@@ -145,54 +137,40 @@ ppc_error_hist <-
         binwidth = binwidth,
         breaks = breaks
       ) +
+      xlab(error_label()) +
       bayesplot_theme_get() +
-      xlab(expression(italic(y) - italic(y)[rep])) +
       dont_expand_y_axis() +
+      error_hist_facets(
+        facet_args,
+        grouped = FALSE,
+        ignore = nrow(yrep) == 1
+      ) +
       force_axes_in_facets() +
       yaxis_title(FALSE) +
       yaxis_text(FALSE) +
       yaxis_ticks(FALSE) +
-      facet_text(FALSE) +
-      facet_bg(FALSE)
+      facet_text(FALSE)
   }
 
 
 #' @rdname PPC-errors
 #' @export
-#' @template args-group
-#'
 ppc_error_hist_grouped <-
   function(y,
            yrep,
            group,
            ...,
+           facet_args = list(),
            binwidth = NULL,
            breaks = NULL,
            freq = TRUE) {
+
     check_ignored_arguments(...)
-
-    y <- validate_y(y)
-    yrep <- validate_yrep(yrep, y)
-    group <- validate_group(group, y)
-    errors <- grouped_error_data(y, yrep, group)
-
-    ggplot(errors, set_hist_aes(freq)) +
-      geom_histogram(
-        fill = get_color("l"),
-        color = get_color("lh"),
-        size = 0.25,
-        binwidth = binwidth,
-        breaks = breaks
-      ) +
-      facet_grid(rep_id ~ group, scales = "free") +
-      bayesplot_theme_get() +
-      xlab(expression(italic(y) - italic(y)[rep])) +
-      dont_expand_y_axis(c(0.005, 0)) +
-      force_axes_in_facets() +
-      yaxis_text(FALSE) +
-      yaxis_ticks(FALSE) +
-      yaxis_title(FALSE) +
-      facet_bg(FALSE) +
+    call <- match.call(expand.dots = FALSE)
+    g <- eval(ungroup_call("ppc_error_hist", call), parent.frame())
+    g +
+      error_hist_facets(facet_args, grouped = TRUE) +
+      facet_text() +
       theme(strip.text.y = element_blank())
   }
 
@@ -203,48 +181,23 @@ ppc_error_scatter <-
   function(y,
            yrep,
            ...,
+           facet_args = list(),
            size = 2.5,
            alpha = 0.8) {
     check_ignored_arguments(...)
 
     y <- validate_y(y)
-    yrep <- validate_yrep(yrep, y)
-
-    if (nrow(yrep) == 1) {
-      return(
-        .ppc_scatter(
-          data = data.frame(y = y, x = y - as.vector(yrep)),
-          mapping = aes_(x = ~ x, y = ~ y),
-          x_lab = expression(italic(y) - italic(y)[rep]),
-          y_lab = expression(italic(y)),
-          size = size,
-          alpha = alpha,
-          abline = FALSE
-        )
-      )
-    }
-
+    yrep <- validate_predictions(yrep, length(y))
     errors <- compute_errors(y, yrep)
-    .ppc_scatter(
-      data = dplyr::left_join(
-        melt_yrep(errors),
-        data.frame(y = y, y_id = seq_along(y)),
-        by = "y_id"
-      ),
-      mapping = aes_(x = ~ value, y = ~ y),
-      y_lab = expression(italic(y)),
-      x_lab = expression(italic(y) - italic(y)[rep]),
+    ppc_scatter(
+      y = y,
+      yrep = errors,
+      facet_args = facet_args,
       size = size,
       alpha = alpha,
-      abline = FALSE
+      ref_line = FALSE
     ) +
-      facet_wrap(
-        facets = ~ rep_id
-        # labeller = label_bquote(italic(y) - italic(y)[rep](.(rep_id)))
-      ) +
-      force_axes_in_facets() +
-      facet_text(FALSE) +
-      facet_bg(FALSE)
+      labs(x = error_label(), y = y_label())
   }
 
 #' @rdname PPC-errors
@@ -258,26 +211,47 @@ ppc_error_scatter_avg <-
     check_ignored_arguments(...)
 
     y <- validate_y(y)
-    yrep <- validate_yrep(yrep, y)
-
-    if (nrow(yrep) == 1)
-      return(
-        ppc_error_scatter(y, yrep,
-                          size = size,
-                          alpha = alpha, ...)
-      )
-
-    .ppc_scatter(
-      data = data.frame(y, avg_error = y - colMeans(yrep)),
-      mapping = aes_(x = ~ avg_error, y = ~ y),
-      y_lab = y_label(),
-      x_lab = "Average predictive error",
-      alpha = alpha,
+    yrep <- validate_predictions(yrep, length(y))
+    errors <- compute_errors(y, yrep)
+    ppc_scatter_avg(
+      y = y,
+      yrep = errors,
       size = size,
-      abline = FALSE
-    )
+      alpha = alpha,
+      ref_line = FALSE
+    ) +
+      labs(x = error_avg_label(), y = y_label())
   }
 
+
+#' @rdname PPC-errors
+#' @export
+ppc_error_scatter_avg_grouped <-
+  function(y,
+           yrep,
+           group,
+           ...,
+           facet_args = list(),
+           size = 2.5,
+           alpha = 0.8) {
+    check_ignored_arguments(...)
+
+    y <- validate_y(y)
+    yrep <- validate_predictions(yrep, length(y))
+    errors <- compute_errors(y, yrep)
+    ppc_scatter_avg_grouped(
+      y = y,
+      yrep = errors,
+      group = group,
+      size = size,
+      alpha = alpha,
+      facet_args = facet_args,
+      ref_line = FALSE
+    ) +
+      labs(x = error_avg_label(), y = y_label())
+  }
+
+
 #' @rdname PPC-errors
 #' @export
 #' @param x A numeric vector the same length as `y` to use as the x-axis
@@ -293,104 +267,159 @@ ppc_error_scatter_avg_vs_x <-
     check_ignored_arguments(...)
 
     y <- validate_y(y)
-    yrep <- validate_yrep(yrep, y)
+    yrep <- validate_predictions(yrep, length(y))
     x <- validate_x(x, y)
-    .ppc_scatter(
-      data = data.frame(x, avg_error = y - colMeans(yrep)),
-      mapping = aes_(x = ~ x, y = ~ avg_error),
-      x_lab = expression(italic(x)),
-      y_lab = "Average predictive error",
-      alpha = alpha,
+    errors <- compute_errors(y, yrep)
+    ppc_scatter_avg(
+      y = x,
+      yrep = errors,
       size = size,
-      abline = FALSE
-    )
+      alpha = alpha,
+      ref_line = FALSE
+    ) +
+      labs(x = error_avg_label(), y = expression(italic(x))) +
+      coord_flip()
   }
 
 
 #' @rdname PPC-errors
 #' @export
 #' @param bins For `ppc_error_binned()`, the number of bins to use (approximately).
-ppc_error_binned <- function(y, yrep, ..., bins = NULL, size = 1, alpha = 0.25) {
-  check_ignored_arguments(...)
+ppc_error_binned <-
+  function(y,
+           yrep,
+           ...,
+           facet_args = list(),
+           bins = NULL,
+           size = 1,
+           alpha = 0.25) {
+    check_ignored_arguments(...)
 
-  y <- validate_y(y)
-  yrep <- validate_yrep(yrep, y)
-  binned <- binned_error_data(y, yrep, bins = bins)
+    data <- ppc_error_binnned_data(y, yrep, bins = bins)
+    facet_layer <- if (nrow(yrep) == 1) {
+      geom_ignore()
+    } else {
+      facet_args[["facets"]] <- "rep_id"
+      do.call("facet_wrap", facet_args)
+    }
 
-  mixed_scheme <- is_mixed_scheme(color_scheme_get())
-  point_fill <- get_color(ifelse(mixed_scheme, "m", "d"))
-  point_color <- get_color(ifelse(mixed_scheme, "mh", "dh"))
-  graph <-
-    ggplot(binned, aes_(x = ~ ey_bar)) +
-    geom_hline(
-      yintercept = 0,
-      linetype = 2,
-      color = "black"
-    ) +
-    geom_ribbon(
-      aes_(ymax = ~ se2, ymin = ~ -se2),
-      fill = get_color("l"),
-      color = NA,
-      alpha = alpha
-    ) +
-    geom_path(
-      mapping = aes_(y = ~ se2),
-      color = get_color("l"),
-      size = size
-    ) +
-    geom_path(
-      mapping = aes_(y = ~ -se2),
-      color = get_color("l"),
-      size = size
-    ) +
-    geom_point(
-      mapping = aes_(y = ~ err_bar),
-      shape = 21,
-      fill = point_fill,
-      color = point_color
-    ) +
-    labs(
-      x = "Predicted proportion",
-      y = "Average Errors \n (with 2SE bounds)"
-    ) +
-    bayesplot_theme_get()
+    mixed_scheme <- is_mixed_scheme(color_scheme_get())
+    point_fill <- get_color(ifelse(mixed_scheme, "m", "d"))
+    point_color <- get_color(ifelse(mixed_scheme, "mh", "dh"))
 
-  if (nrow(yrep) > 1) {
-    graph <- graph +
-      facet_wrap(
-        facets = ~rep_id
-        # labeller = label_bquote(italic(y)[rep](.(rep_id)))
-      )
+    ggplot(data, aes_(x = ~ ey_bar)) +
+      hline_0(linetype = 2, color = "black") +
+      geom_ribbon(
+        mapping = aes_(ymax = ~ se2, ymin = ~ -se2),
+        fill = get_color("l"),
+        color = NA,
+        alpha = alpha
+      ) +
+      geom_path(
+        mapping = aes_(y = ~ se2),
+        color = get_color("l"),
+        size = size
+      ) +
+      geom_path(
+        mapping = aes_(y = ~ -se2),
+        color = get_color("l"),
+        size = size
+      ) +
+      geom_point(
+        mapping = aes_(y = ~ err_bar),
+        shape = 21,
+        fill = point_fill,
+        color = point_color
+      ) +
+      labs(
+        x = "Predicted proportion",
+        y = "Average Errors \n (with 2SE bounds)"
+      ) +
+      bayesplot_theme_get() +
+      facet_layer +
+      force_axes_in_facets() +
+      facet_text(FALSE)
   }
 
-  graph +
-    force_axes_in_facets() +
-    facet_text(FALSE) +
-    facet_bg(FALSE)
+
+#' @rdname PPC-errors
+#' @export
+ppc_error_data <- function(y, yrep, group = NULL) {
+  y <- validate_y(y)
+  yrep <- validate_predictions(yrep, length(y))
+  if (!is.null(group)) {
+    group <- validate_group(group, length(y))
+  }
+  errors <- compute_errors(y, yrep) %>% melt_predictions()
+  errors <- tibble::add_column(errors, y_obs = y[errors$y_id], .before = "rep_id")
+  if (!is.null(group)) {
+    errors <- tibble::add_column(errors, group = group[errors$y_id], .before = "y_id")
+  }
+  errors
 }
 
 
 # internal ----------------------------------------------------------------
+
+#' Compute predictive errors `y` - `yrep`
+#' @noRd
+#' @param y,yrep User's `y` and `yrep` arguments.
+#' @return A matrix with the same dimensions as `yrep`
 compute_errors <- function(y, yrep) {
-  errs <- sweep(yrep, MARGIN = 2L, STATS = as.array(y), FUN = "-")
-  as.matrix(-1 * errs)
+  suggested_package("rstantools")
+  rstantools::predictive_error(object = yrep, y = y)
 }
 
-grouped_error_data <- function(y, yrep, group) {
-  grps <- unique(group)
-  errs <- list()
-  for (j in seq_along(grps)) {
-    g_j <- grps[j]
-    err_j <- compute_errors(y[group == g_j], yrep[, group == g_j, drop=FALSE])
-    errs[[j]] <- melt_yrep(err_j)
-    errs[[j]]$group <- g_j
+
+#' Create facet layer for PPC error plots
+#'
+#' The default is to use `scales="fixed"` (which I think makes sense for looking
+#' at errors, right?) if not specified in `facet_args`.
+#'
+#' @param User's `facet_args` argument.
+#' @param grouped If `FALSE` then does faceting by `rep_id`, if `TRUE` then both
+#'   `rep_id` and `group`.
+#' @param ignore If `TRUE` then `geom_ignore()` is returned. This is intended to
+#'   allow turning off facets if there is only one plot to make.
+#' @param scales_default What to use for the `scales` argument to `facet_*()` if
+#'   not specified in `facet_args`.
+#' @return Object returned by `facet_wrap()` or `facet_grid()` (unless `ignore=TRUE`).
+#' @noRd
+error_hist_facets <-
+  function(facet_args,
+           grouped = FALSE,
+           ignore = FALSE,
+           scales_default = "fixed") {
+    if (ignore) {
+      return(geom_ignore())
+    }
+
+    if (grouped) {
+      facet_fun <- "facet_grid"
+      facet_args[["facets"]] <- rep_id ~ group
+    } else {
+      facet_fun <- "facet_wrap"
+      facet_args[["facets"]] <- ~ rep_id
+    }
+    facet_args[["scales"]] <- facet_args[["scales"]] %||% scales_default
+
+    do.call(facet_fun, facet_args)
   }
-  dat <- dplyr::bind_rows(errs)
-  dat$y_id <- NULL
-  return(dat)
+
+
+error_label <- function() {
+  expression(italic(y) - italic(y)[rep])
+}
+error_avg_label <- function() {
+  expression(paste("Average ", italic(y) - italic(y)[rep]))
 }
 
-binned_error_data <- function(y, yrep, bins = NULL) {
+
+# Data for binned errors plots
+ppc_error_binnned_data <- function(y, yrep, bins = NULL) {
+  y <- validate_y(y)
+  yrep <- validate_predictions(yrep, length(y))
+
   if (is.null(bins)) {
     bins <- n_bins(length(y))
   }
@@ -398,11 +427,17 @@ binned_error_data <- function(y, yrep, bins = NULL) {
   errors <- compute_errors(y, yrep)
   binned_errs <- list()
   for (s in 1:nrow(errors)) {
-    binned_errs[[s]] <- bin_errors(ey = yrep[s,], r = errors[s,], bins = bins,
-                                   rep_id = s)
+    binned_errs[[s]] <-
+      bin_errors(
+        ey = yrep[s, ],
+        r = errors[s, ],
+        bins = bins,
+        rep_id = s
+      )
   }
-  dat <- dplyr::bind_rows(binned_errs)
-  return(dat)
+
+  binned_errs <- dplyr::bind_rows(binned_errs)
+  tibble::as_tibble(binned_errs)
 }
 
 # calculate number of bins binned_error_data()
