@@ -72,14 +72,15 @@
 #'                + (1 + floor | county),
 #'   data = radon,
 #'   iter = 1000,
-#'   chains = 2  # ,cores = 2
+#'   chains = 2,
+#'   cores = 2
 #'  )
 #' y <- radon$log_radon
 #' yrep <- posterior_predict(fit)
 #'
-#' loo1 <- loo(fit, save_psis = TRUE, cores = 2)
+#' loo1 <- loo(fit, save_psis = TRUE, cores = 4)
 #' psis1 <- loo1$psis_object
-#' lw <- weights(psis1)
+#' lw <- weights(psis1) # normalized log weights
 #'
 #' # marginal predictive check using LOO probability integral transform
 #' color_scheme_set("orange")
@@ -234,17 +235,17 @@ ppc_loo_pit_overlay <- function(y,
       )
   }
 
-    p +
-      scale_color_ppc_dist(labels = c("PIT", "Unif")) +
-      scale_y_continuous(
-        limits = c(0, NA),
-        expand = expansion(mult = c(0, .25))
-      ) +
-      bayesplot_theme_get() +
-      yaxis_title(FALSE) +
-      xaxis_title(FALSE) +
-      yaxis_text(FALSE) +
-      yaxis_ticks(FALSE)
+  p +
+    scale_color_ppc(labels = c("PIT", "Unif")) +
+    scale_y_continuous(
+      limits = c(0, NA),
+      expand = expansion(mult = c(0, .25))
+    ) +
+    bayesplot_theme_get() +
+    yaxis_title(FALSE) +
+    xaxis_title(FALSE) +
+    yaxis_text(FALSE) +
+    yaxis_ticks(FALSE)
 }
 
 #' @rdname PPC-loo
@@ -265,7 +266,7 @@ ppc_loo_pit_data <-
     } else {
       suggested_package("rstantools")
       y <- validate_y(y)
-      yrep <- validate_yrep(yrep, y)
+      yrep <- validate_predictions(yrep, length(y))
       stopifnot(identical(dim(yrep), dim(lw)))
       pit <- rstantools::loo_pit(object = yrep, y = y, lw = lw)
     }
@@ -290,7 +291,6 @@ ppc_loo_pit_data <-
     data
   }
 
-
 #' @rdname PPC-loo
 #' @export
 ppc_loo_pit_qq <- function(y,
@@ -310,7 +310,7 @@ ppc_loo_pit_qq <- function(y,
   } else {
     suggested_package("rstantools")
     y <- validate_y(y)
-    yrep <- validate_yrep(yrep, y)
+    yrep <- validate_predictions(yrep, length(y))
     stopifnot(identical(dim(yrep), dim(lw)))
     pit <- rstantools::loo_pit(object = yrep, y = y, lw = lw)
   }
@@ -369,8 +369,6 @@ ppc_loo_pit <-
   }
 
 
-
-
 #' @rdname PPC-loo
 #' @export
 #' @template args-prob-prob_outer
@@ -408,8 +406,9 @@ ppc_loo_intervals <-
            ...,
            prob = 0.5,
            prob_outer = 0.9,
+           alpha = 0.33,
            size = 1,
-           fatten = 3,
+           fatten = 2.5,
            order = c("index", "median")) {
 
     check_ignored_arguments(...)
@@ -426,7 +425,7 @@ ppc_loo_intervals <-
       }
     } else {
       suggested_package("loo", min_version = "2.0.0")
-      yrep <- validate_yrep(yrep, y)
+      yrep <- validate_predictions(yrep, length(y))
       if (!is.null(subset)) {
         stopifnot(length(y) >= length(subset))
         y <- y[subset]
@@ -448,24 +447,46 @@ ppc_loo_intervals <-
     if (order_by_median) {
       x <- reorder(x, intervals[, 2])
     }
-
-    graph <- .ppc_intervals(
-      data = .loo_intervals_data(y, x, intervals),
-      grouped = FALSE,
-      style = "intervals",
-      size = size,
-      fatten = fatten,
-      x_lab = "Data point (index)"
-    )
-
-    if (!order_by_median) {
-      return(graph)
+    xy_labs <- if (order_by_median) {
+      labs(x = "Ordered by median", y = NULL)
+    } else {
+      intervals_axis_labels(has_x = FALSE)
     }
 
-    graph +
-      xlab("Ordered by median") +
-      xaxis_text(FALSE) +
-      xaxis_ticks(FALSE)
+    data <- .loo_intervals_data(y, x, intervals)
+    ggplot(data) +
+      intervals_inner_aes(
+        needs_y = TRUE,
+        color = "yrep",
+        fill = "yrep"
+      ) +
+      geom_linerange(
+        mapping = intervals_outer_aes(color = "yrep"),
+        alpha = alpha,
+        size = size
+      ) +
+      geom_pointrange(
+        shape = 21,
+        stroke = 0.5,
+        size = size,
+        fatten = fatten
+      ) +
+      geom_point(
+        mapping = aes_(
+          y = ~ y_obs,
+          color = "y",
+          fill = "y"
+        ),
+        shape = 21,
+        stroke = 0.5,
+        size = 1
+      ) +
+      scale_color_ppc() +
+      scale_fill_ppc() +
+      bayesplot_theme_get() +
+      xy_labs +
+      xaxis_text(!order_by_median) +
+      xaxis_ticks(!order_by_median)
   }
 
 #' @rdname PPC-loo
@@ -495,7 +516,7 @@ ppc_loo_ribbon <-
       }
     } else {
       suggested_package("loo", min_version = "2.0.0")
-      yrep <- validate_yrep(yrep, y)
+      yrep <- validate_predictions(yrep, length(y))
       if (!is.null(subset)) {
         stopifnot(length(y) >= length(subset))
         y <- y[subset]
@@ -512,14 +533,38 @@ ppc_loo_ribbon <-
         probs = sort(c(a, 0.5, 1 - a))
       )$value))
     }
-    .ppc_intervals(
-      data = .loo_intervals_data(y, x = seq_along(y), intervals),
-      grouped = FALSE,
-      style = "ribbon",
-      size = size,
-      alpha = alpha,
-      x_lab = "Data point (index)"
-    )
+
+    data <- .loo_intervals_data(y, x = seq_along(y), intervals)
+    ggplot(data) +
+      intervals_inner_aes(fill = "yrep", color = "yrep") +
+      geom_ribbon(
+        mapping = intervals_outer_aes(fill = "yrep", color = "yrep"),
+        alpha = alpha,
+        size = 0.05
+      ) +
+      geom_ribbon(
+        mapping = intervals_outer_aes(),
+        alpha = 1,
+        size = 0.05,
+        fill = NA,
+        color = get_color("m")
+      ) +
+      geom_ribbon(size = 0.05) +
+      geom_line(
+        mapping = aes_(y = ~ m),
+        color = get_color("m"),
+        size = size
+      ) +
+      geom_blank(aes_(fill = "y")) +
+      geom_line(
+        aes_(y = ~ y_obs, color = "y"),
+        size = 0.5,
+        alpha = 2/3
+      ) +
+      scale_color_ppc() +
+      scale_fill_ppc(values = c(NA, get_color("l"))) +
+      intervals_axis_labels(has_x = FALSE) +
+      bayesplot_theme_get()
   }
 
 
@@ -528,7 +573,7 @@ ppc_loo_ribbon <-
 .loo_intervals_data <- function(y, x, intervals) {
   stopifnot(length(y) == nrow(intervals), length(x) == length(y))
 
-  data.frame(
+  tibble::tibble(
     y_id = seq_along(y),
     y_obs = y,
     x = x,
@@ -540,6 +585,7 @@ ppc_loo_ribbon <-
 }
 
 # subset a psis_object without breaking it
+# (FIXME: use function from loo package when subset.psis() method becomes available)
 .psis_subset <- function(psis_object, subset) {
   stopifnot(all(subset == as.integer(subset)))
   if (length(subset) > dim(psis_object)[2]) {
