@@ -3,14 +3,22 @@ context("Aesthetics")
 
 
 # color scheme stuff ------------------------------------------------------
+
+prepare_colors_for_test <- function(scheme) {
+  setNames(
+    bayesplot:::master_color_list[[scheme]],
+    bayesplot:::scheme_level_names()
+  )
+}
+
 test_that("getting and setting the color scheme works", {
   color_scheme_set("red")
-  expect_equivalent(color_scheme_get(), prepare_colors("red"))
-  expect_named(prepare_colors("blue"), scheme_level_names())
+  expect_equivalent(color_scheme_get(), prepare_colors_for_test("red"))
+  expect_named(prepare_colors_for_test("blue"), scheme_level_names())
   expect_named(color_scheme_get(), scheme_level_names())
   for (clr in names(master_color_list)) {
     color_scheme_set(clr)
-    expect_equivalent(color_scheme_get(), prepare_colors(clr),
+    expect_equivalent(color_scheme_get(), prepare_colors_for_test(clr),
                       info = clr)
     expect_named(color_scheme_get(), scheme_level_names())
   }
@@ -20,7 +28,10 @@ test_that("getting and setting the color scheme works", {
   expect_gg(plot(color_scheme_get("mix-blue-green")))
 
   color_scheme_set("blue")
-  expect_equivalent(color_scheme_get("teal"), prepare_colors("teal"))
+  expect_equivalent(color_scheme_get("teal"), prepare_colors_for_test("teal"))
+
+  # error if not character
+  expect_error(color_scheme_set(7), "'scheme' should be a character vector of length 1 or 6")
 })
 
 test_that("color_scheme_get with i argument works", {
@@ -54,10 +65,17 @@ test_that("setting mixed scheme works", {
   color_scheme_set("mix-blue-gray")
   expect_equivalent(color_scheme_get(), mixed_scheme("blue", "gray"))
 
-  expect_error(color_scheme_set("mix-green-reds"),
-               "should be one of")
-  expect_error(color_scheme_set("mix-greens-red"),
-               "should be one of")
+  expect_error(color_scheme_set("mix-green-reds"), "should be one of")
+  expect_error(color_scheme_set("mix-greens-red"), "should be one of")
+})
+
+test_that("setting brewer scheme works", {
+  skip_if_not_installed("RColorBrewer")
+  color_scheme_set("brewer-Blues")
+  expect_equivalent(unlist(color_scheme_get()), RColorBrewer::brewer.pal(6, "Blues"))
+  color_scheme_set("brewer-Spectral")
+  expect_equivalent(unlist(color_scheme_get()), RColorBrewer::brewer.pal(6, "Spectral"))
+  expect_error(color_scheme_set("brewer-FAKE"), "FAKE is not a valid palette")
 })
 
 orange_scheme_bad <-
@@ -75,6 +93,8 @@ test_that("color_scheme_set throws correct errors for custom schemes ", {
                "not found: not_a_color1, not_a_color2")
   expect_error(color_scheme_set(c("red", "blue")),
                "should be a character vector of length 1 or 6")
+  expect_error(prepare_custom_colors(c("red", "blue")),
+               "Custom color schemes must contain exactly 6 colors")
 })
 
 test_that("mixed_scheme internal function doesn't error", {
@@ -97,7 +117,7 @@ test_that("get_color returns correct color values", {
   scheme <- color_scheme_set("green")
   levs <- scheme_level_names()
 
-  ans <- unlist(prepare_colors("green")[levs], use.names = FALSE)
+  ans <- unlist(prepare_colors_for_test("green")[levs], use.names = FALSE)
   expect_identical(get_color(levs), ans)
   for (lev in levs)
     expect_identical(get_color(lev), scheme[[lev]], info = lev)
@@ -178,3 +198,33 @@ test_that("ggplot2::theme_set overrides bayesplot theme", {
   ggplot2::theme_set(minimal)
   expect_identical(bayesplot_theme_get(), minimal)
 })
+
+bayesplot_theme_set(bayesplot::theme_default())
+color_scheme_set()
+
+
+
+# Visual tests ------------------------------------------------------------
+
+test_that("color_scheme_view renders correctly", {
+  testthat::skip_on_cran()
+  testthat::skip_if_not_installed("vdiffr")
+
+  color_scheme_set()
+  p_default <- color_scheme_view()
+  vdiffr::expect_doppelganger("color_scheme_view (default)", p_default)
+
+  p_red <- color_scheme_view("red")
+  vdiffr::expect_doppelganger("color_scheme_view (scheme specified)", p_red)
+
+  p_mix <- color_scheme_view("mix-red-blue")
+  vdiffr::expect_doppelganger("color_scheme_view (mixed scheme)", p_mix)
+
+  p_brewer <- color_scheme_view("brewer-Spectral")
+  vdiffr::expect_doppelganger("color_scheme_view (brewer palette)", p_brewer)
+
+  color_scheme_set()
+})
+
+bayesplot_theme_set(bayesplot::theme_default())
+color_scheme_set()
